#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <uv.h>

#include "http.h"
#include "player.h"
#include "server.h"
#include "udp.h"

const int BACKLOG = 128;
const int PORT = 26950;

// TODO: on sigterm, cleanly shut down by sending shutdown packets to everyone
// (double check if this is useful first)

void basic_alloc(uv_handle_t* handle, size_t suggested_size, uv_buf_t* buf) {
  (void)handle;
  buf->base = malloc(suggested_size);
  buf->len = suggested_size;
}

int main(void) {
#ifndef _WIN32
  // Pls no crasherino
  signal(SIGPIPE, SIG_IGN);
#endif

  uv_loop_t* loop = uv_default_loop();
  http_server = malloc(sizeof(uv_tcp_t));
  hollowknight_server = malloc(sizeof(uv_udp_t));
  start_tms = time(NULL);

  // HTTP Listen
  {
    struct sockaddr_in addr;
    uv_tcp_init(loop, http_server);
    uv_ip4_addr("0.0.0.0", HTTP_PORT, &addr);
    uv_tcp_bind(http_server, (const struct sockaddr*)&addr, 0);
    int err =
        uv_listen((uv_stream_t*)http_server, BACKLOG, new_http_connection);
    if (err) {
      fprintf(stderr, "Failed to listen: %s\n", uv_strerror(err));
      return 1;
    }
    printf("[HTTP] Listening on 0.0.0.0:%d...\n", HTTP_PORT);
  }

  // UDP Listen
  {
    struct sockaddr_in addr;
    uv_udp_init_ex(loop, hollowknight_server, UV_UDP_RECVMMSG);
    uv_ip4_addr("0.0.0.0", PORT, &addr);
    uv_udp_bind(hollowknight_server, (const struct sockaddr*)&addr,
                UV_UDP_REUSEADDR);
    uv_udp_recv_start(hollowknight_server, basic_alloc, udp_read);
    printf("[UDP] Listening on 0.0.0.0:%d...\n", PORT);
  }

  // Timer to kick inactive clients
  uv_timer_t timer_req;
  uv_timer_init(loop, &timer_req);
  uv_timer_start(&timer_req, kick_idle_players, 1000 * IDLE_PLAYER_TIMEOUT,
                 1000);

  // Tick timer
  uv_timer_t timer_tick;
  uv_timer_init(loop, &timer_tick);
  uv_timer_start(&timer_tick, process_tick, 0, 1000 / TICKRATE);

  int res = uv_run(loop, UV_RUN_DEFAULT);
  free(http_server);
  free(hollowknight_server);
  return res;
}
