#pragma once
#include <sys/time.h>
#include <uv.h>

#include "protocol.h"
#include "server.h"

typedef struct unacked_packet_s {
  // We can store the player pointer in a structure that gets passed to libuv,
  // because we cancel the timer before freeing the player.
  Player* player;

  unsigned char resends;
  unsigned short sequence;
  uv_timer_t timer;
  uv_buf_t buffer;

  struct timespec send_time;
} UnackedPacket;

typedef struct player_s {
  // Networking
  struct sockaddr_storage addr;
  time_t last_packet_tms;
  unsigned short local_sequence_number;
  unsigned short ack;
  unsigned int ack_field;
  uv_buf_t* must_send_packets;
  uv_buf_t* may_send_packets;
  int nb_must_send_packets;
  int nb_may_send_packets;
  int outgoing_bytes;
  int nb_unacked_packets;
  UnackedPacket* unacked_packets;
  unsigned short rtt;

  // Settings
  unsigned short id;
  char username[MAX_STRING_LENGTH];
  unsigned char team;
  unsigned char skin_id;
  bool is_scene_host;

  // Info
  char scene[MAX_STRING_LENGTH];
  unsigned short animation;
  float x, y;
  float map_x, map_y;
  bool scale;
} Player;

void end_udp_write(uv_udp_send_t* req, int status);
void end_write(uv_write_t* req, int status);
void kick_player(Player* player, bool is_timeout);

extern Player players[MAX_PLAYERS];
