#pragma once
#include <limits.h>
#include <stdbool.h>
#include <uv.h>

// Maximum string length, including the null terminator.
// HKMP allows strings up to USHRT_MAX, but let's not.
#define MAX_STRING_LENGTH 200

// Sent from server to client
typedef enum server_packet_e {
  S_PACKET_LOGIN = 0,
  S_PACKET_CONNECT,
  S_PACKET_DISCONNECT,
  S_PACKET_SERVER_SHUTDOWN,
  S_PACKET_ENTER_SCENE,
  S_PACKET_IN_SCENE,
  S_PACKET_LEAVE_SCENE,
  S_PACKET_UPDATE_PLAYER,
  S_PACKET_UPDATE_ENTITY,
  S_PACKET_DIE,
  S_PACKET_UPDATE_TEAM,
  S_PACKET_UPDATE_SKIN,
  S_PACKET_UPDATE_SETTINGS,
} ServerPacketType;

// Sent from client to server
typedef enum client_packet_e {
  C_PACKET_LOGIN = 0,
  C_PACKET_HELLO,
  C_PACKET_DISCONNECT,
  C_PACKET_UPDATE_PLAYER,
  C_PACKET_UPDATE_ENTITY,
  C_PACKET_ENTER_SCENE,
  C_PACKET_LEAVE_SCENE,
  C_PACKET_DIE,
  C_PACKET_UPDATE_TEAM,
  C_PACKET_UPDATE_SKIN,
} ClientPacketType;

typedef enum {
  ENTITY_UPDATE_POSITION = 0,
  ENTITY_UPDATE_STATE,
  ENTITY_UPDATE_VARIABLES,
} EntityUpdateType;

typedef enum {
  PLAYER_UPDATE_POSITION = 0,
  PLAYER_UPDATE_SCALE,
  PLAYER_UPDATE_MAP_POSITION,
  PLAYER_UPDATE_ANIMATION
} PlayerUpdateType;

// Reads n bytes and copies them to out_ptr.
// out_ptr needs to be already allocated.
#define READ_BYTES(out_ptr, n)    \
  if (length < n) return pos + n; \
  memcpy(out_ptr, data + pos, n); \
  length -= n;                    \
  pos += n

#define READ_BYTE(var) READ_BYTES(&var, 1)
#define READ_SHORT(var) READ_BYTES(&var, 2)
#define READ_INT(var) READ_BYTES(&var, 4)
#define READ_FLOAT(var) READ_BYTES(&var, 4)

#define READ_BOOL(var)  \
  do {                  \
    unsigned char byte; \
    READ_BYTE(byte);    \
    var = byte > 0;     \
  } while (0)

// Reads a string to out_ptr.
// out_ptr needs to be already allocated.
// The macro ensures it doesn't exceed MAX_STRING_LENGTH - 1 bytes.
#define READ_STRING(out_ptr)                                               \
  do {                                                                     \
    unsigned short string_length;                                          \
    READ_SHORT(string_length);                                             \
    if (string_length > MAX_STRING_LENGTH - 1) return pos + string_length; \
    READ_BYTES(out_ptr, string_length);                                    \
    out_ptr[string_length] = '\0';                                         \
  } while (0)

#define WRITE_BYTES(bytes, n)         \
  memcpy(packet + out_pos, bytes, n); \
  out_pos += n

#define WRITE_BYTE(byte)         \
  do {                           \
    unsigned char lvalue = byte; \
    WRITE_BYTES(&lvalue, 1);     \
  } while (0)

#define WRITE_BOOL(val)      \
  do {                       \
    bool lvalue = val;       \
    WRITE_BYTES(&lvalue, 1); \
  } while (0)

#define WRITE_FLOAT(num)     \
  do {                       \
    float lvalue = num;      \
    WRITE_BYTES(&lvalue, 2); \
  } while (0)

#define WRITE_SHORT(num)         \
  do {                           \
    unsigned short lvalue = num; \
    WRITE_BYTES(&lvalue, 2);     \
  } while (0)

#define WRITE_INT(num)       \
  do {                       \
    int lvalue = num;        \
    WRITE_BYTES(&lvalue, 4); \
  } while (0)

#define WRITE_STRING(str)                   \
  do {                                      \
    unsigned short write_len = strlen(str); \
    WRITE_SHORT(write_len);                 \
    WRITE_BYTES(str, write_len);            \
  } while (0)

// static inline void deferred_free(void *p) {
//         free(*(void**) p);
// }
// #define _free_on_scope_exit_ __attribute__((cleanup(deferred_free)))

#define INIT_PACKET(packet_type)                                   \
  int out_pos = 0;                                                 \
  __attribute__((cleanup(free))) char* packet = malloc(USHRT_MAX); \
  WRITE_INT(packet_type)

typedef struct player_s Player;
int handle_packet(Player* player, char* data, int length);
