#include "server.h"

#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "player.h"
#include "protocol.h"
#include "udp.h"

int start_tms = 0;

// This method assumes you already set player->scene to the new scene.
void add_player_to_scene(Player* player) {
  int players_in_scene = 0;

  // Send ENTER_SCENE to all players in the scene
  {
    INIT_PACKET(S_PACKET_ENTER_SCENE);
    WRITE_SHORT(player->id);
    WRITE_STRING(player->username);
    WRITE_FLOAT(player->x);
    WRITE_FLOAT(player->y);
    WRITE_BOOL(player->scale);
    WRITE_BYTE(player->team);
    WRITE_BYTE(player->skin_id);
    WRITE_SHORT(player->animation);

    for (int i = 0; i < MAX_PLAYERS; i++) {
      if (players[i].id != player->id &&
          (strcmp(players[i].scene, player->scene) == 0)) {
        players_in_scene++;
        IMPORTANT_PACKET_SEND(&players[i]);
      }
    }
  }

  // Send scene info to player that just joined
  {
    INIT_PACKET(S_PACKET_IN_SCENE);

    if (players_in_scene > 255) players_in_scene = 255;
    WRITE_BYTE(players_in_scene);

    for (int i = 0; i < MAX_PLAYERS; i++) {
      if (players[i].id != player->id &&
          (strcmp(players[i].scene, player->scene) == 0)) {
        WRITE_SHORT(players[i].id);
        WRITE_STRING(players[i].username);
        WRITE_FLOAT(players[i].x);
        WRITE_FLOAT(players[i].y);
        WRITE_BOOL(players[i].scale);
        WRITE_BYTE(players[i].team);
        WRITE_BYTE(players[i].skin_id);
        WRITE_SHORT(players[i].animation);
      }
    }

    if (players_in_scene == 0) {
      player->is_scene_host = true;
    }
    WRITE_BOOL(player->is_scene_host);

    IMPORTANT_PACKET_SEND(player);
  }
}

void remove_player_from_scene(Player* player) {
  if (player->scene[0] == '\0') {
    return;
  }

  INIT_PACKET(S_PACKET_LEAVE_SCENE);
  WRITE_SHORT(player->id);
  IMPORTANT_SCENE_BROADCAST(player);

  player->scene[0] = '\0';
  player->is_scene_host = false;

  // TODO: add host rotation, i don't think that's done by HKMP?
}

// Kick players idle for more than IDLE_PLAYER_TIMEOUT seconds
void kick_idle_players(uv_timer_t* handle) {
  (void)handle;

  time_t current_tms = time(NULL);
  for (int i = 0; i < MAX_PLAYERS; i++) {
    if (players[i].username[0] == '\0') continue;
    if (players[i].last_packet_tms + IDLE_PLAYER_TIMEOUT < current_tms) {
      kick_player(&players[i], 1);
    }
  }
}