#pragma once
#include <uv.h>

// There can only be 255 players in a scene, so let's not allow more
#define MAX_PLAYERS 255

// How many updates per second should we send to each player?
// NOTE: We define an 'update' as a group of packets totalling <= 508 bytes,
// not as individual packets.
#define TICKRATE 64

// Maximum number of bytes to be sent per tick to each player. If a packet is
// defined as 'important', it will be sent even if it goes over the limit.
#define MAX_BYTES_PER_TICK 508

// Maximum number of unacked 'important' packets before the player gets kicked.
#define MAX_UNACKED_PACKETS 100

// Maximum number of times an 'important' packets is resent when unacked.
#define MAX_RESENDS 10

#define IDLE_PLAYER_TIMEOUT 5

// Game settings
#define PVP_ENABLED 0
#define BODY_DAMAGE_ENABLED 0
#define ALWAYS_SHOW_MAP_ICONS 0
#define ONLY_SHOW_MAP_ICONS_WITH_WAYWARD_COMPASS 1
#define DISPLAY_NAMES 1
#define TEAMS_ENABLED 0
#define ALLOW_SKINS 1
#define NAIL_DAMAGE 1
#define GRUBBERFLY_ELEGY_DAMAGE 1
#define VENGEFUL_SPIRIT_DAMAGE 1
#define SHADE_SOUL_DAMAGE 2
#define DESOLATE_DIVE_DAMAGE 1
#define HOWLING_WRAITH_DAMAGE 1
#define ABYSS_SHRIEK_DAMAGE 2
#define GREAT_SLASH_DAMAGE 2
#define DASH_SLASH_DAMAGE 2
#define CYCLONE_SLASH_DAMAGE 1
#define SPORE_SHROOM_DAMAGE 1
#define SPORE_DUNG_SHROOM_DAMAGE 1
#define THORN_OF_AGONY_DAMAGE 1

typedef struct player_s Player;
void add_player_to_scene(Player* player);
void remove_player_from_scene(Player* player);
void kick_idle_players(uv_timer_t* handle);

extern int start_tms;
